#!/bin/csh
#
#   Control script to submit multiple jobs on a cluster using
#   the Sun Grid Engine. Each job processes N particles. N is
#   specified in the 'mparameters' file as 'increment'.
#
cp mparameters mparameters_run

set start = `grep start_process mparameters_run | awk '{print $2}'`
set end   = `grep end_process mparameters_run | awk '{print $2}'`
set first = `grep first_particle mparameters_run | awk '{print $2}'`
set last  = `grep last_particle mparameters_run | awk '{print $2}'`
set incr  = `grep increment mparameters_run | awk '{print $2}'`
set data_input = `grep data_input mparameters_run | awk '{print $2}'`
set working_directory = `pwd`
set SCRATCH = ../scratch

mainloop:

cp mparameters mparameters_run
\rm ${data_input}_$start.par >& /dev/null

# queue submission loop

set firstn = $first
@ lastn = $first + $incr - 1

while ( $lastn <= $last )
  echo "Submitted particles from" $firstn to $lastn on `date`
  qsub -l mem_free=1G -cwd -r y -V -e ~/stderr -o ~/stderr -N a${firstn}_$lastn \
	$working_directory/msearch_n.com $firstn $lastn $start
  #sleep 3
  if ( $lastn == $last ) then
    set firstn = $first
    @ lastn = $first + $incr - 1
    goto checkdone
  endif
  @ firstn = $firstn + $incr
  @ lastn = $lastn + $incr
  if ( $firstn >= $last ) set firstn = $last
  if ( $lastn >= $last ) set lastn = $last
end

checkdone:

sleep 5
\rm ~/stderr
while ( $lastn <= $last )
  set continue = 0

  grep "overall phase residual" $SCRATCH/${data_input}.par_${firstn}_$lastn >& /dev/null
  if (! $status) then
    echo "Particles $firstn to $lastn finished....  "`date`
    if ($firstn == $first ) head -60 $SCRATCH/${data_input}.par_${firstn}_${lastn} | grep C >> ${data_input}_$start.par
    grep -v C $SCRATCH/${data_input}.par_${firstn}_${lastn} >> ${data_input}_$start.par
    \rm $SCRATCH/${data_input}.par_${firstn}_${lastn} >& /dev/null
    if ( $lastn == $last ) goto reconstruct
    set continue = 1
  endif
  if (! $continue ) goto checkdone
  @ firstn = $firstn + $incr
  @ lastn = $lastn + $incr
  if ( $firstn >= $last ) set firstn = $last
  if ( $lastn >= $last ) set lastn = $last
end

reconstruct:
echo "Calculating 3D structure...."
\rm $SCRATCH/${data_input}_mreconstruct.log
\rm $SCRATCH/${data_input}.shft_*
\rm $SCRATCH/${data_input}_mrefine_n.log_*
qsub -l mem_free=1.0G -cwd -r y -V -e $SCRATCH/stderr -o $SCRATCH/stderr -N r${first}_${last} ${working_directory}/mreconstruct.com $first $last $start

checkdoner:

sleep 5

grep --binary-files=text "mreconstruct.com finished" $SCRATCH/${data_input}_mreconstruct.log >& /dev/null
if ($status) goto checkdoner 

cat $SCRATCH/${data_input}.res >> ${data_input}_$start.par

sleep 10

if ($start < $end ) then
  @ start = $start + 1
  goto mainloop
endif
date
