C**************************************************************************
      SUBROUTINE CTFAPPLY_PHASE_ONLY(NSAM,SPEC,SPEQ,SHX,SHY,CS,WL,WGH1,
     +		 WGH2,DFMID1,DFMID2,ANGAST,THETATR,CTFF,CTFS,OUTD,
     +		 OUTC,OUTQ,AMAGP,RIH,HALFW,RI2,RI3,RI4,
     +		 DATD,DATC,DATQ,B3DV,PHI,THETA,PSI,W,XSTD,
     +           IBUF,TX,TY,ASYM)
C**************************************************************************
C  Applies calculated CTF to a particle transform and also masks the 
C  particle in real space with a cosine-bell-edged circle of radius RI
C  Calls rlft3 and MASKENV.
C  Uses function CTF.
C  Used in CALCFX.
C**************************************************************************
      IMPLICIT NONE

      INTEGER NSAM,JC,L,LL,M,MM,ID,I,II,NMID,ISUM,NSAMH,IBUF,IC
      REAL PHASE,OUTD(*),FI1,AMAGP,CRAD2,SRAD2,RIH,W(*)
      REAL RI2,RI3,HALFW,PSHFTR,DATD(*),PI,TX,TY
      REAL RI4,SHX,SHY,CS,WL,WGH1,WGH2,DFMID1,DFMID2,ANGAST
      REAL THETATR,THETATRP
      REAL RIHP,SCAL
      REAL B3DV(*),PHI,THETA,PSI,XSTD
      PARAMETER  (PI=3.1415926535897)
      COMPLEX CTF,CTFV,CTFR,CTFL,CTFF(*),CTFS(*)
      COMPLEX PSHFT,SPEC(*),SPEQ(*),OUTC(*)
      COMPLEX OUTQ(*),DATC(*),DATQ(*)
      CHARACTER ASYM*3
C**************************************************************************
C      write(*,*)'Now inside CTFAPPLY',AMAGP
      THETATRP=THETATR/AMAGP
      RIHP=RIH/AMAGP
      SCAL=NSAM*NSAM/2
C	  THETATRP is the diffraction angle for the first pixel in 
C	  the transform of a particle sampled at the input pixel size
      JC=NSAM/2+1
      NSAMH=NSAM/2
      NMID=NSAM/2+1
      IC=NSAM*NSAM/2
C
C     CORRECT FOR CTF AND APPLY SHIFTS
      DO 111 L=1,JC
        LL=L-1
        DO 111 M=1,NSAM
          MM=M-1
          IF (MM.GE.JC) MM=MM-NSAM
          PHASE=SHX*LL+SHY*MM
          PSHFT=CMPLX(COS(PHASE),SIN(PHASE))
          CTFR=CTF(CS,WL,WGH1,WGH2,DFMID1,DFMID2,
     +             ANGAST,THETATRP,LL,MM,TX,TY)
          CTFL=CTF(CS,WL,WGH1,WGH2,DFMID1,DFMID2,
     +             ANGAST,THETATRP,-LL,-MM,TX,TY)
C 	in this unweighted CTF-correction subroutine, the value of 
C 	CTFV can only be either +1.0 or -1.0
          CTFV=CTFR+CONJG(CTFL)
      	  IF(ABS(CTFV).NE.0.0) CTFV=CTFV/ABS(CTFV)
          IF (L.NE.JC) THEN
      	    ID=L+NSAMH*(M-1)
            OUTC(ID)=SPEC(ID)*CTFV*PSHFT
            DATC(ID)=SPEC(ID)*PSHFT
      	    CTFF(ID)=CTFR
      	    CTFF(ID+IC)=CTFL
          ELSE
      	IF(M.LT.0.OR.M.GT.NSAM) WRITE(*,*)'M=',M
            OUTQ(M)=SPEQ(M)*CTFV*PSHFT
            DATQ(M)=SPEQ(M)*PSHFT
      	    CTFS(M)=CTFR
      	    CTFS(M+NSAM)=CTFL
          ENDIF
111   CONTINUE

C      write(*,*)'before RLFT3s'
      CALL RLFT3(OUTD,OUTQ,NSAM,NSAM,1,-1)
      DO 21 I=1,NSAM*NSAM
        OUTD(I)=OUTD(I)/SCAL
C        DATD(I)=OUTD(I)
21    CONTINUE

C     MASK PARTICLE, COSINE EDGE
      IF (ASYM(1:1).EQ.'H') THEN
        CALL MASKCOS_C(NSAM,OUTD,RI2,RI3,RIH,
     +                 HALFW,AMAGP,PSI)
      ELSE
        CALL MASKCOS(NSAM,OUTD,RI2,RI3,RIH,
     +               HALFW,AMAGP)
      ENDIF

      IF (XSTD.LT.0.0) THEN
C	Apply tight mask to image used for refinement.
      	CALL MASKENV(NSAM,RIHP,OUTD,B3DV,0.0,0.0,
     +               PHI,THETA,PSI,W,IBUF)
      ENDIF

C      CALL RLFT3(DATD,DATQ,NSAM,NSAM,1,1)
      CALL RLFT3(OUTD,OUTQ,NSAM,NSAM,1,1)
C      write(*,*)'after RLFT3s'

C     SHIFT PARTICLE TO ORIGIN
      DO 110 L=1,JC
        LL=L-1
        DO 110 M=1,NSAM
          MM=M-1
          IF (MM.GE.JC) MM=MM-NSAM
      	  ISUM=(LL+MM)
      	  PSHFTR=1.0
      	  IF (MOD(ISUM,2).NE.0) PSHFTR=-1.0
          IF (L.NE.JC) THEN
      	    ID=L+NSAMH*(M-1)
            DATC(ID)=DATC(ID)*PSHFTR
            OUTC(ID)=OUTC(ID)*PSHFTR
          ELSE
            DATQ(M)=DATQ(M)*PSHFTR
            OUTQ(M)=OUTQ(M)*PSHFTR
          ENDIF
110   CONTINUE

      RETURN
      END
