C**************************************************************************
      SUBROUTINE D3MASK(NSAM,C3DV,B3DV,B3DF,B3DS,XSTD)
C**************************************************************************
C When XSTD is positive (see CALL), produces 5-pixel-cosine-bell smoothed 
C mask boundary defined by density being above the mean level of a low pass 
C filtered map by an amount equal to XSTD times the standard deviation of 
C the map.

C Calls rlft3.  
C Used by MASK.
C**************************************************************************
      IMPLICIT NONE

      INTEGER NSAM,L,LL,M,MM,N,NN,ID,IS,I,J,K,NSAMH,IRAD,JC,NSAM3
      INTEGER NM
      REAL RAD2,FRAD,B3DV(*),TLEVEL,XSTD,EDGE,PI,C3DV(*)
      PARAMETER (FRAD=0.1)
      PARAMETER (IRAD=5)
      PARAMETER (PI=3.1415926535897)
      DOUBLE PRECISION DMEAN,DSTD
      COMPLEX B3DF(*),B3DS(*)
C**************************************************************************
      JC=NSAM/2+1
      NSAMH=NSAM/2
      NSAM3=NSAM*NSAM*NSAM

      DO 10 I=1,NSAM3
      	B3DV(I)=C3DV(I)
10    CONTINUE
      CALL RLFT3(B3DV,B3DS,NSAM,NSAM,NSAM,1)

C     Low-pass filter 3D map....
      DO 88 L=1,JC
        LL=L-1
        DO 88 M=1,NSAM
          MM=M-1
          IF (MM.GE.JC) MM=MM-NSAM
          DO 88 N=1,NSAM
            NN=N-1
            IF (NN.GE.JC) NN=NN-NSAM
            RAD2=REAL(LL**2+MM**2+NN**2)/NSAM**2/FRAD**2
            IF (L.NE.JC) THEN
              ID=L+NSAMH*((M-1)+NSAM*(N-1))
              B3DF(ID)=B3DF(ID)*EXP(-RAD2)
            ELSE
              IS=M+NSAM*(N-1)
              B3DS(IS)=B3DS(IS)*EXP(-RAD2)
            ENDIF
88    CONTINUE

      CALL RLFT3(B3DV,B3DS,NSAM,NSAM,NSAM,-1)

C     Calculate mean, STD
      DSTD=0.0D0
      DMEAN=0.0D0
      DO 87 I=1,NSAM3
        DMEAN=DMEAN+B3DV(I)
        DSTD=DSTD+B3DV(I)**2
87    CONTINUE
      DMEAN=DMEAN/NSAM3
      DSTD=DSTD/NSAM3
      DSTD=DSTD-DMEAN**2
      IF (DSTD.GT.0.0D0) THEN
        DSTD=SQRT(DSTD)
      ELSE
        DSTD=0.0D0
      ENDIF

C     Threshold filtered map using XSTD x STD above mean
      TLEVEL=DMEAN+ABS(XSTD)*DSTD
      DO 86 I=1,NSAM3
        IF (B3DV(I).GE.TLEVEL) THEN
          B3DV(I)=1.0
        ELSE
          B3DV(I)=0.0
        ENDIF
86    CONTINUE

C     Smoothing the edge.....
      DO 11 I=1,NSAM
      	DO 11 J=1,NSAM
      	  DO 11 K=1,NSAM
      	    ID=K+NSAM*((J-1)+NSAM*(I-1))
      	    IF (B3DV(ID).EQ.1.0) THEN
      	      DO 12 L=-IRAD,IRAD
      	        DO 12 M=-IRAD,IRAD
      	          DO 12 N=-IRAD,IRAD
      	            RAD2=L**2+M**2+N**2
      	            RAD2=SQRT(RAD2)
                    EDGE=(1.0+COS(PI*RAD2/IRAD))/2.0
      		    ID=K+N+NSAM*((J+M-1)+NSAM*(I+L-1))
      	            IF ((ID.GE.1).AND.(ID.LE.NSAM3)
     +			.AND.(RAD2.LE.IRAD)) THEN
      	              IF (B3DV(ID).LT.EDGE) B3DV(ID)=EDGE
      	            ENDIF
12	      CONTINUE
      	    ENDIF
11    CONTINUE

C     Calculate mean within edge
      DMEAN=0.0D0
      NM=0
      DO 21 I=1,NSAM3
        IF ((B3DV(I).LT.1.0).AND.(B3DV(I).GT.0.0)) THEN
          DMEAN=DMEAN+C3DV(I)
          NM=NM+1
        ENDIF
21    CONTINUE
      IF (NM.NE.0) THEN
        DMEAN=DMEAN/NM
      ENDIF

C      CALL RLFT3(C3DV,C3DS,NSAM,NSAM,NSAM,-1)
      DO 20 I=1,NSAM3
C      	C3DV(I)=C3DV(I)*B3DV(I)/NSAM3
      	C3DV(I)=C3DV(I)*B3DV(I)+(1.0-B3DV(I))*DMEAN
20    CONTINUE
C      CALL RLFT3(C3DV,C3DS,NSAM,NSAM,NSAM,1)

      RETURN
      END
